/*
 * Copyright (c) 2015, 2016, 2017 Adrian Siekierka
 *
 * This file is part of Preston.
 *
 * Preston is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Preston is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Preston.  If not, see <http://www.gnu.org/licenses/>.
 */

package pl.asie.preston.recipe;

import net.minecraft.inventory.InventoryCrafting;
import net.minecraft.item.ItemStack;
import net.minecraft.item.crafting.IRecipe;
import net.minecraft.world.World;
import net.minecraftforge.common.crafting.IShapedRecipe;
import net.minecraftforge.registries.IForgeRegistryEntry;
import pl.asie.preston.PrestonMod;
import pl.asie.preston.util.PrestonUtils;
import pl.asie.preston.container.ItemCompressedBlock;

public class RecipeCompress extends IForgeRegistryEntry.Impl<IRecipe> implements IShapedRecipe {
	@Override
	public boolean func_77569_a(InventoryCrafting inv, World worldIn) {
		int stackCount = 0;
		for (int i = 0; i < inv.func_174922_i() * inv.func_174923_h(); i++) {
			ItemStack stack = inv.func_70463_b(i % inv.func_174922_i(), i / inv.func_174922_i());
			if (!stack.func_190926_b()) {
				stackCount++;
			}
		}

		if (stackCount != getRecipeWidth() * getRecipeHeight()) {
			return false;
		}

		for (int yShift = 0; yShift <= inv.func_174923_h() - getRecipeHeight(); yShift++) {
			for (int xShift = 0; xShift <= inv.func_174922_i() - getRecipeWidth(); xShift++) {
				boolean found = true;
				ItemStack cmpStack = null;

				for (int i = 0; i < getRecipeWidth() * getRecipeHeight(); i++) {
					ItemStack stack = inv.func_70463_b(xShift + i % getRecipeWidth(), yShift + i / getRecipeWidth());
					if (stack.func_190926_b()) {
						found = false;
						break;
					} else if (!ItemCompressedBlock.canCompress(stack) || (cmpStack != null && !PrestonUtils.canMerge(cmpStack, stack))) {
						return false;
					} else {
						cmpStack = stack;
					}
				}

				if (found) {
					return true;
				}
			}
		}

		return false;
	}

	@Override
	public ItemStack func_77572_b(InventoryCrafting inv) {
		for (int yShift = 0; yShift <= inv.func_174923_h() - getRecipeHeight(); yShift++) {
			for (int xShift = 0; xShift <= inv.func_174922_i() - getRecipeWidth(); xShift++) {
				ItemStack stack = inv.func_70463_b(xShift, yShift);
				if (!stack.func_190926_b()) {
					return ItemCompressedBlock.shiftLevel(stack, 1);
				}
			}
		}

		return ItemStack.field_190927_a;
	}

	@Override
	public boolean func_194133_a(int width, int height) {
		return width >= getRecipeWidth() && height >= getRecipeHeight();
	}

	@Override
	public ItemStack func_77571_b() {
		return ItemStack.field_190927_a;
	}

	@Override
	public boolean func_192399_d() {
		return true;
	}

	@Override
	public int getRecipeWidth() {
		return PrestonMod.COMPRESSED_BLOCK_AMOUNT == 4 ? 2 : 3;
	}

	@Override
	public int getRecipeHeight() {
		return PrestonMod.COMPRESSED_BLOCK_AMOUNT == 4 ? 2 : 3;
	}
}
