/*
 * Copyright (c) 2015, 2016, 2017 Adrian Siekierka
 *
 * This file is part of Preston.
 *
 * Preston is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Preston is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Preston.  If not, see <http://www.gnu.org/licenses/>.
 */

package pl.asie.preston.util;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;

import java.util.ArrayList;
import java.util.Collection;

public abstract class ContainerBase extends Container {
	public final InventoryPlayer playerInventory;
	protected final Collection<Slot> SLOTS_PLAYER = new ArrayList<>(36);
	protected final Collection<Slot> SLOTS_INVENTORY = new ArrayList<>();
	private final IContainerHandler containerHandler;

	public ContainerBase(InventoryPlayer inventoryPlayer) {
		this(inventoryPlayer, null);
	}

	public ContainerBase(InventoryPlayer inventoryPlayer, IContainerHandler listener) {
		this.containerHandler = listener;
		this.playerInventory = inventoryPlayer;
		if (listener != null) {
			listener.onOpenedBy(inventoryPlayer.field_70458_d);
		}
	}

	@Override
	public boolean func_75145_c(EntityPlayer player) {
		return containerHandler != null ? containerHandler.isUsableByPlayer(player) : true;
	}

	@Override
	public ItemStack func_82846_b(EntityPlayer player, int slot) {
		Slot slotObject = field_75151_b.get(slot);
		if (slotObject != null && slotObject.func_75216_d()) {
			ItemStack stack = tryTransferStackInSlot(player, slotObject, slotObject.field_75224_c == player.field_71071_by ? SLOTS_INVENTORY : SLOTS_PLAYER);
			func_75142_b();
			return stack;
		} else {
			return slotObject.func_75211_c();
		}
	}

	private boolean tryInsertStackToSlot(EntityPlayer player, Slot from, Slot to) {
		if (!to.func_75216_d() || PrestonUtils.canMerge(from.func_75211_c(), to.func_75211_c())) {
			ItemStack fromStack = from.func_75211_c();
			ItemStack toStack = to.func_75211_c();

			int maxSize = Math.min(toStack.func_77976_d(), to.func_75219_a());
			if (toStack.func_190926_b()) {
				int amount = Math.min(fromStack.func_190916_E(), maxSize);

				if (amount > 0) {
					to.func_75215_d(fromStack.func_77979_a(amount));
					to.func_75215_d(to.func_75211_c());

					return true;
				}
			} else {
				int amount = Math.min(maxSize - toStack.func_190916_E(), fromStack.func_190916_E());

				if (amount > 0) {
					fromStack.func_190918_g(amount);
					toStack.func_190917_f(amount);
					to.func_75215_d(to.func_75211_c());

					return true;
				}
			}
		}

		return false;
	}

	// TODO: BUGTEST ME
	protected ItemStack tryTransferStackInSlot(EntityPlayer player, Slot from, Collection<Slot> targets) {
		ItemStack fromStack = from.func_75211_c();
		Collection<Slot> targetsValidEmpty = new ArrayList<>(targets.size());
		boolean dirty = false;

		if (!from.func_75216_d())
			return from.func_75211_c();

		// Pass 1: Merge
		for (Slot to : targets) {
			if (to.func_75214_a(fromStack)) {
				if (to.func_75216_d()) {
					dirty |= tryInsertStackToSlot(player, from, to);

					if (!from.func_75216_d())
						break;
				} else {
					targetsValidEmpty.add(to);
				}
			}
		}

		// Pass 2: Place
		if (from.func_75216_d()) {
			for (Slot to : targetsValidEmpty) {
				dirty |= tryInsertStackToSlot(player, from, to);

				if (!from.func_75216_d())
					break;
			}
		}

		if (dirty) {
			// Using putStack instead of onSlotChanged here,
			// as putStack calls the latter + IItemHandler's
			// onContentsChanged.
			from.func_75215_d(from.func_75211_c());
		}

		return fromStack;
	}

	public void bindPlayerInventory(InventoryPlayer inventoryPlayer, int startX, int startY) {
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 9; j++) {
				func_75146_a(new Slot(inventoryPlayer, j + i * 9 + 9,
						startX + j * 18, startY + i * 18));
			}
		}
		for (int i = 0; i < 9; i++) {
			func_75146_a(new Slot(inventoryPlayer, i, startX + i * 18, startY + 58));
		}
	}

	@Override
	protected Slot func_75146_a(Slot slotIn) {
		slotIn = super.func_75146_a(slotIn);
		if (slotIn.field_75224_c instanceof InventoryPlayer) {
			SLOTS_PLAYER.add(slotIn);
		} else {
			SLOTS_INVENTORY.add(slotIn);
		}
		return slotIn;
	}

	@Override
	public void func_75134_a(EntityPlayer player) {
		super.func_75134_a(player);
		if (containerHandler != null) {
			this.containerHandler.onClosedBy(player);
		}
	}
}
